/***************************************************************************
 *   Copyright (C) 2004 by Zhang Wei                                       *
 *   brep@163.com                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qaction.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmenubar.h>
#include <qmessagebox.h>

#include "mainwindow.hxx"
#include "fontsheet.hxx"

MainWindow::MainWindow(QWidget *parent, const char *name)
	: QMainWindow(parent, name)
{
	fontSheet = new FontSheet(this);
	setCentralWidget(fontSheet);

	connect(fontSheet, SIGNAL(modified()), this, SLOT(fontsheetModified()));
	
	createActions();
	createMenus();

	setCaption("Zhcon BPSF Font File Editor");
	
	fileFilter = ("Zhcon BPSF Font File (*.bpsf)");
	
	modified = false;
}

void MainWindow::createActions()
{
	newAct = new QAction(tr("&New"), tr("Ctrl+N"), this);
	connect(newAct, SIGNAL(activated()), this, SLOT(newFile()));

	openAct = new QAction(tr("&Open"), tr("Ctrl+O"), this);
	connect(openAct, SIGNAL(activated()), this, SLOT(open()));

	saveAct = new QAction(tr("&Save"), tr("Ctrl+S"), this);
	connect(saveAct, SIGNAL(activated()), this, SLOT(save()));

	saveAsAct = new QAction(tr("Save &As"), tr("Ctrl+A"), this);
	connect(saveAsAct, SIGNAL(activated()), this, SLOT(saveAs()));

	exitAct = new QAction(tr("E&xit"), tr("Ctrl+E"), this);
	connect(exitAct, SIGNAL(activated()), this, SLOT(close()));

	usageAct = new QAction(tr("&Usage"), tr("Ctrl+U"), this);
	connect(usageAct, SIGNAL(activated()), this, SLOT(usage()));

	aboutAct = new QAction(tr("A&bout"), tr("Ctrl+B"), this);
	connect(aboutAct, SIGNAL(activated()), this, SLOT(about()));
}

void MainWindow::createMenus()
{
	fileMenu = new QPopupMenu(this);
	newAct->addTo(fileMenu);
	openAct->addTo(fileMenu);
	saveAct->addTo(fileMenu);
	saveAsAct->addTo(fileMenu);
	fileMenu->insertSeparator();
	exitAct->addTo(fileMenu);


	helpMenu = new QPopupMenu(this);
	usageAct->addTo(helpMenu);
	aboutAct->addTo(helpMenu);

	menuBar()->insertItem("&File", fileMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem("&Help", helpMenu);
}

void MainWindow::newFile()
{
	if(maybeSave()) {
		fontSheet->clear();
	}
}

bool MainWindow::maybeSave()
{
	if(modified) {
		int ret = QMessageBox::
			warning(this, "BPSF Editor",
				("The font file has been modified.\n"
				 "Do you want to save your changes?"),
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape);
		if (ret == QMessageBox::Yes)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void MainWindow::open()
{
	if (maybeSave()) {
		QString fileName =
			QFileDialog::getOpenFileName(".", fileFilter, this);
		if(!fileName.isEmpty())
			loadFile(fileName);
	}
}

void MainWindow::loadFile(const QString &fileName)
{
	if (fontSheet->readFile(fileName)) 
		setCurrentFile(fileName);
}

bool MainWindow::save()
{
	if(curFile.isEmpty()) {
		return saveAs();
		
	}
	else {
		saveFile(curFile);
		return true;
	}
}

void MainWindow::saveFile(const QString& fileName)
{
	if(fontSheet->writeFile(fileName))
		setCurrentFile(fileName);
}

bool MainWindow::saveAs()
{
	QString fileName =
		QFileDialog::getSaveFileName(".", fileFilter, this);
	if(fileName.isEmpty())
		return false;

	if(QFile::exists(fileName)) {
		int ret = QMessageBox::
			warning(this, tr("BPSF Editor"),
				tr("File %1 already exists.\n"
				 "Do you want to overwrite it?")
				.arg(QDir::convertSeparators(fileName)),
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape);
		if (ret == QMessageBox::No)
			return true;
	}
	if(!fileName.isEmpty())
		saveFile(fileName);
	return true;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
	if (maybeSave()) {
		event->accept();
	}
	else {
		event->ignore();
	}
}

void MainWindow::setCurrentFile(const QString &fileName)
{
	curFile = fileName;
	modified = false;

	if(curFile.isEmpty()) {
		setCaption(tr("Zhcon BPSF font file editor."));
	}
	else {
		setCaption(tr("%1 - %2").arg(strippedName(curFile))
			   .arg(tr("BPSF editor")));
	}
}

QString MainWindow::strippedName(const QString &fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}

void MainWindow::fontsheetModified()
{
	modified = true;
}

void MainWindow::usage()
{
	QMessageBox
		::about(this, tr("Usage"),
			tr("<p>Double click a Font Cell to Popup the Font Editor.</p>"
			 "<p>Use the <b>Left Mouse Button</b> as a pencil.</p>"
			 "<p>Use the <b>Right Mouse Button</b> as a eraser.</p>"
			 "<p>Enjoy it!</p>"));
	
}

void MainWindow::about()
{
	
	QMessageBox
		::about(this, tr("About BPSF Editor"),
			tr("<h2>BPSF font file editor.</h2>"
			 "<p>Copyright &copy; 2005 by Zhang Wei</p>"
			 "<p>brep@163.com</p>"
			 "<p>This editor is a small appliction that "
			 "dedicated to edit the Zhcon's BPSF font file."));
}

		
		     


	
